<?php
if (! class_exists ( 'WC_Braintree_Payment_Gateway' )) {
	return;
}
/**
 * Abstract class that should be extended by local payment gateways.
 *
 * @version 3.0.0
 * @package Braintree/Abstracts
 *         
 */
abstract class WC_Braintree_Local_Payment_Gateway extends WC_Braintree_Payment_Gateway {

	/**
	 *
	 * @var array currencies that this payment method accepts
	 */
	protected $currencies = array();

	public $default_title = '';

	public function __construct() {
		$this->template = 'local-payment.php';
		$this->token_type = 'Local_Payment';
		parent::__construct ();
		$this->settings[ 'method_format' ] = 'source_and_payer';
		// must always be capture per Braintree's documentation
		$this->settings[ 'charge_type' ] = 'capture';
	}

	public function add_hooks() {
		parent::add_hooks ();
		remove_filter ( 'wc_braintree_admin_settings_tabs', array( 
				$this, 'admin_settings_tabs' 
		) );
		add_filter ( 'wc_braintree_local_gateways_tab', array( 
				$this, 'admin_settings_tabs' 
		) );
		add_action ( 'woocommerce_settings_checkout_braintree_local_gateways_' . $this->id, array( 
				$this, 'admin_options' 
		) );
		add_action ( 'wc_braintree_settings_before_options_braintree_local_gateways_' . $this->id, array( 
				$this, 'navigation_menu' 
		) );
		add_action ( 'woocommerce_update_options_checkout_braintree_local_gateways_' . $this->id, array( 
				$this, 'process_admin_options' 
		) );
	}

	public function set_supports() {
		$this->supports = array( 'products', 'refunds', 
				'pre-orders', 'wc_braintree_fees' 
		);
	}

	public function init_form_fields() {
		$this->form_fields = apply_filters ( 'wc_' . $this->id . '_form_fields', include WC_BRAINTREE_PATH . 'includes/gateways/settings/local-payment-settings.php', $this );
	}

	/**
	 * Return true only if the gateway requirements for being displayed are met.
	 *
	 * @return bool
	 */
	public function is_local_payment_available() {
		$customer = WC ()->customer;
		return in_array ( $customer->get_billing_country (), $this->countries ) && in_array ( get_woocommerce_currency (), $this->currencies );
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Braintree_Payment_Gateway::enqueue_checkout_scripts()
	 */
	public function enqueue_checkout_scripts($scripts) {}

	public function localize_script_params() {
		return array_merge ( $this->get_localized_standard_params (), array( 
				'payment_type' => str_replace ( 'braintree_', '', $this->id ), 
				'button_text' => $this->order_button_text, 
				'return_url' => wc_get_checkout_url (), 
				'routes' => array( 
						'payment_data' => braintree ()->rest_api->local_payment->rest_url () . 'payment-data' 
				) 
		) );
	}

	public function output_settings_nav() {
		global $current_section, $wc_braintree_subsection;
		parent::output_settings_nav ();
		include braintree ()->plugin_path () . 'includes/admin/views/html-local-gateways-nav.php';
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Braintree_Payment_Gateway::get_payment_method_from_transaction()
	 */
	public function get_payment_method_from_transaction($transaction) {
		return $transaction->localPayment;
	}

	public function get_gateway_supports_description() {
		$text = '';
		if ($this->currencies) {
			$text = sprintf ( __ ( '%s is available when store currency is %s', 'woo-payment-gateway' ), $this->get_title (), '<b>' . implode ( ', ', $this->currencies ) . '</b>' );
		}
		if ($this->countries) {
			$text .= sprintf ( __ ( ' and billing country is %s', 'woo-payment-gateway' ), '<b>' . implode ( ', ', $this->countries ) . '</b>' );
		}
		return $text;
	}

	public function process_payment($order_id) {
		$order = wc_get_order ( $order_id );
		// check that the payment wasn't processed using webhooks
		if ($order->get_transaction_id () || ! $order->needs_payment ()) {
			return array( 'result' => 'success', 
					'redirect' => $order->get_checkout_order_received_url () 
			);
		}
		return parent::process_payment ( $order_id );
	}
}