<?php
/**
 * Controller class that is called when a local payment gateway is used.
 * This controller stores
 * the payer data for later use by webhooks so the payment can be completed.
 *
 * @since 3.0.0
 * @package Braintree/API
 * @author User
 *        
 */
class WC_Braintree_Controller_Local_Payment extends WC_Braintree_Rest_Controller {

	protected $namespace = 'local-payment/';

	public function register_routes() {
		register_rest_route ( $this->rest_uri (), 'payment-data', array( 
				'methods' => WP_REST_Server::CREATABLE, 
				'callback' => array( $this, 
						'save_payment_data' 
				) 
		) );
		// this is a test route
		/*
		 * register_rest_route ( $this->rest_uri (), 'test', array(
		 * 'methods' => WP_REST_Server::CREATABLE,
		 * 'callback' => array( $this, 'process'
		 * )
		 * ) );
		 */
	}

	/**
	 *
	 * @param WP_REST_Request $request        	
	 */
	public function process($request) {
		$gateway = braintree ()->gateway ();
		$result = $gateway->transaction ()->sale ( array( 
				'amount' => 1, 
				'options' => [ 
						'submitForSettlement' => True 
				], 
				'paypalAccount' => [ 
						'paymentId' => $request->get_param ( 'paymentId' ), 
						'payerId' => $request->get_param ( 'payerId' ) 
				] 
		) );
	}

	/**
	 * Store the payer Id against the order so it can be used later on to process the payment.
	 *
	 * @param WP_REST_Request $request        	
	 */
	public function save_payment_data($request) {
		try {
			$checkout = WC ()->checkout ();
			// get the order id;
			$order_id = $checkout->create_order ( array( 
					'payment_method' => $request->get_param ( 'payment_method' ) 
			) );
			if (is_wp_error ( $order_id )) {
				throw new Exception ( $order_id->get_error_message () );
			}
			// set the order awaiting payment parameter so the same order will be processed for the payment_id.
			WC ()->session->set ( 'order_awaiting_payment', $order_id );
			$order = wc_get_order ( $order_id );
			$order->update_meta_data ( 'wc_braintree_payment_id', $request->get_param ( 'payment_id' ) );
			$order->save ();
			return rest_ensure_response ( array( 
					'order_id' => $order_id 
			) );
		} catch ( Exception $e ) {
			return new WP_Error ( 'order-error', $e->getMessage (), array( 
					'status' => 200 
			) );
		}
	}
}